=begin
	RENDERING FAVOURITES PLUGIN
	v1.1, 09 December 2012
	by Trog Luddite
	
	Stores and recalls up to 8 favourite view rendering settings.
	Adds store and recall items to the View menu, plus toolbar.
	Menu items are shortcut key assignable.
	Note that this is not switching styles, but modifying the model's
	rendering setting directly, so stores some settings not available
	by switching styles - e.g. hide rest of model settings, shadow settings.
	To use...
		1) Set up a model appearance using the regular controls
		2) Choose one of the 'Store' items from the menu or toolbar
		3) Choose a 'Recall' item to recall a previously stored appearance
		
	Other functions
	
	Export renders
		Recalls each stored favourite in turn, and exports a bitmap file
	of the model with each render appearance.  Files are saved to a folder
	within the current model file's directory called "<model name> Renders".
	Each file is named "<model name>_<favourite number>_<favourite name>.<ext>"
	
	Set Options
		Allow the user to set custom options for render recall and export...
		- Fog/Shadows.  You can choose to recall all fog/shadow settings, not
		recall them at all (disable), or recall only their on/off status.
		- Export File options.  As per a normal bitmap export.
		- Export from-to.  Choose to export only a subset of the current
		render favourites (empty favourite slots will be skipped).
		
	Render Information
		Displays a pop-up box showing the currently stored favourites, and
	a complete listing of all the current options being used to
	render the current model.
	
	About
		Displays a pop-up box showing info about the plugin and a brief
	user guide.
	
=end

require 'sketchup.rb'

module Trog

	# Set maximum number of memories
	# Note cannot be more than 8, as there are not icons for the toolbar
	# for a greater number.
	MAX_RENDERS = 8
	RENDER_OPTS_KEYS = [:fog,:shadows,:file_type,:width,:height,:antialias,:compression,:transparent,:from,:to]
		
	
	# Set file paths and make a directory if there isn't one yet
	RENDERS_DIR = File.join(File.dirname(__FILE__), 'Trog_Renders')
	RENDERS_PATH = File.join(RENDERS_DIR, 'Trog_Renders.dump')
	RENDERS_OPTS_PATH = File.join(RENDERS_DIR,'Trog_Renders.opt')
	Dir.mkdir(RENDERS_DIR) unless File.exist?(RENDERS_DIR)

	# Load last used settings at startup
	if File.exists?(RENDERS_PATH)
		@@renders,@@render_plugin_opts = File.open(RENDERS_PATH){|file| Marshal.load(file)}
	else # Create an empty favourites file if none exists
		@@renders = Array.new(MAX_RENDERS)
		@@render_plugin_opts = {
			:fog => 'Status Only',
			:shadows => 'Status Only',
			:file_type => '.png',
			:height => Sketchup.active_model.active_view.vpheight,
			:width => Sketchup.active_model.active_view.vpwidth,
			:antialias => 'Enabled',
			:compression => 0.9,
			:transparent => 'Disabled',
			:from => 1,
			:to => MAX_RENDERS}
		File.open(RENDERS_PATH,'w') {|file| Marshal.dump([@@renders,@@render_plugin_opts],file)}
	end
	
	def self.renders_to_hash
	# Return the current rendering options and shadow info as nested
	# hashes amd arrays so that they can be marshalled.
		hashes = [{},{},'']
		# hashes[0] = renderinf options hash
		render_opts = Sketchup.active_model.rendering_options
		render_opts.each do |key,value|
			# Colour cannot be marshalled - convert to array form
			if value.is_a?(Sketchup::Color)
				hashes[0][key] = value.to_a
			else
				hashes[0][key] = value
			end
		end
		# hashes[1] = shadow info hash
		shadow_opts = Sketchup.active_model.shadow_info
		shadow_opts.each do |key,value|
			# Vector3d cannot be marshalled - convert to array form
			if value.is_a?(Geom::Vector3d)
				hashes[1][key] = value.to_a
			else
				hashes[1][key] = value
			end
		end
		return hashes
	end #renders_to_hash
	
	def self.hash_to_renders(hashes)
	# Set the current model's rendering options to the values
	# stored in a regular hash.
	# Returns the rendering_options and shadow_info objects.
		render_opts = Sketchup.active_model.rendering_options
		hashes[0].each do |key,value|
			if !(key.include?('Fog')) || @@render_plugin_opts[:fog]=='All Settings' || (@@render_plugin_opts[:fog]=='Status Only' && key=='DisplayFog')
				# Restore ARGB arrays to sketchup colours
				if value.is_a?(Array)
					render_opts[key] = Sketchup::Color.new(value)
				else
					render_opts[key] = value
				end
			end
		end
		shadow_opts = Sketchup.active_model.shadow_info
		if @@render_plugin_opts[:shadows] == 'All Settings'
			hashes[1].each do |key,value|
				# Restore Vector3d objects from array form
				if value.is_a?(Array)
					shadow_opts[key] = Geom::Vector3d.new(value) 
				else
					shadow_opts[key] = value
				end
			end
		elsif @@render_plugin_opts[:shadows] == 'Status Only'
			shadow_opts['DisplayShadows'] = hashes[1]['DisplayShadows']
		end
		return [render_opts,shadow_opts]
	end #hash to renders
	
	def self.dump_renders
	# Save the array of render settings to file
		File.open(RENDERS_PATH,'w') {|file| Marshal.dump([@@renders,@@render_plugin_opts],file)}
	end #dump_renders
	
	def self.render_exists?(index)
	# Check that a favourites slot contains a render
		return false unless (0...MAX_RENDERS)===index
		return false if @@renders[index].nil?
		true
	end #render_exists?
	
	def self.render_store(index)
	# Store the current render settings into a favourites slot.
	# Returns true/false to indicate success.
		if (0...MAX_RENDERS)===index
			# Ask for a name, or cancel the operation
			prompts = ['Name:']
			defaults = ["favourite #{index+1}"]
			title = "Render favourite #{index+1}"
			input = UI.inputbox(prompts,defaults,title)
			if input
				opts = renders_to_hash
				name = input[0]
				@@renders[index] = opts
				@@renders[index][2] = name
				description = "Recall render #{index+1}: #{name}"
				@@recall_cmds[index].status_bar_text = description
				@@recall_cmds[index].tooltip = description
				dump_renders
			end
		else
			return false
		end
	end #render_store
	
	def self.render_load(index)
	# Read the render settings from a favourites slot.
	# Returns the rendering_options object if succesful, otherwise nil.
		if render_exists?(index)
			opts = hash_to_renders(@@renders[index])
			Sketchup.active_model.active_view.refresh
			return opts
		else
			return nil
		end
	end #render_load
	
	def self.show_renders
	# Displays the current render settings in a message box
		string = "Current favourites...\n"
		(0...MAX_RENDERS).each do |index|
			string << "\nFavourite #{index}: "
			if render_exists?(index)
				string << "#{@@renders[index][2]}"
			else
				string << "<empty>"
			end
		end
		opts = renders_to_hash
		string << "\n\nCurrent rendering options...\n"
		opts[0].each do |key,value|
			if value.is_a?(Array)
				string << "\n#{key} = Color A:#{value[0]}, R:#{value[1]}, G:#{value[2]}, B:#{value[3]}"
			else
				string << "\n#{key} = #{value}"
			end
		end
		opts = Sketchup.active_model.shadow_info
		string << "\n\nCurrent shadow info...\n"
		opts.each do |key,value|
			string << "\n#{key} = #{value}"
		end
		UI.messagebox(string,MB_MULTILINE)
		return nil
	end #show_renders
	
	def self.renders_plugin_opts
	# Options GUI for plugin user settings
	# saved to HDD for memorizing between sessions
		prompts = ['Recall Fog',
			'Recall Shadows',
			'Export File Type',
			'Export File Width',
			'Export File Height',
			'Export File Antialias',
			'Export File Compression',
			'Export File Transparency',
			'Export First Render',
			'Export Last Render']
		defaults = []
		RENDER_OPTS_KEYS.each {|key| defaults << @@render_plugin_opts[key]}
		lists = ["All Settings|Status Only|Disabled",
			"All Settings|Status Only|Disabled",
			".png|.jpg",
			"",
			"",
			"Enabled|Disabled",
			"",
			"Enabled|Disabled",
			"",
			""]
		title = "Render Favourites Options"
		inputs = UI.inputbox(prompts,defaults,lists,title)
		if inputs
			RENDER_OPTS_KEYS.each {|key| @@render_plugin_opts[key] = inputs.shift}
			from = @@render_plugin_opts[:from]
			to = @@render_plugin_opts[:to]
			from = 1 if from < 1
			to = 1 if to < 1
			from = MAX_RENDERS if from > MAX_RENDERS
			to = MAX_RENDERS if to > MAX_RENDERS
			from,to = [from,to].sort
			@@render_plugin_opts[:from] = from
			@@render_plugin_opts[:to] = to
			dump_renders
		end
		return @@render_plugin_opts
	end #renders_plugin_opts
	
	def self.renders_export
	# Export a bitmap image of the model for each stored render favourite
		model = Sketchup.active_model
		model_path = model.path
		model_dir = File.dirname(model_path)
		model_name = File.basename(model_path,'.skp')
		view = model.active_view
		((@@render_plugin_opts[:from]-1)..(@@render_plugin_opts[:to]-1)).each do |index|
			if render_exists?(index)
				render_load(index)
				new_name = model_name + "_#{index+1}_#{@@renders[index][2]}#{@@render_plugin_opts[:file_type]}"
				new_dir = File.join(model_dir,"#{model_name} Renders")
				new_path = File.join(new_dir,new_name)
				Dir.mkdir(new_dir) unless File.exist?(new_dir)
				keys = {
					:filename => new_path,
					:width => @@render_plugin_opts[:width],
					:height => @@render_plugin_opts[:height],
					:antialias => (@@render_plugin_opts[:antialias]=='Enabled'),
					:compression => @@render_plugin_opts[:compression],
					:transparent => (@@render_plugin_opts[:transparent]=='Enabled')}
				view.write_image(keys)
			end
		end
	end #renders_export
	
	def self.render_opts_save
		cwd = Dir.getwd
		Dir.chdir(RENDERS_DIR)
		path = UI.savepanel("Save render favourites",'*.rend')
		if path
			File.open(path,'w'){|file| Marshal.dump([@@renders,@@render_plugin_opts],file)}
		end
		DIR.chdir(cwd)
	end #render_opts_save

	def self.render_opts_load
		cwd = Dir.getwd
		Dir.chdir(RENDERS_DIR)
		path = UI.openpanel("Load render favourites",'*.rend')
		if path
			@@renders,@@render_plugin_opts = File.open(path){|file| Marshal.load(file)}
			dump_renders
		end
		Dir.chdir(cwd)
	end #render_opts_load
	
	def self.about_renders
		string = <<END
Render Favourites v1.1
09 Dec 2012, (c) Trog Luddite

Stores and recalls favourite rendering
presets.  Unlike styles or scenes, this
includes all settings related to model
appearance; including whether to hide
rest of model/components when editing,
all shadow settings etc.
You can also automatically save a folder
of bitmap images - one for each render
favourite.
All menu items can be assigned to
keyboard shortcuts.

To use...

1) Get the model looking the way you like.

2) Choose a 'Store favourite' item from 
the View menu or toolbar.

3) You will be prompted for a name for
your settings.  This will appear as the
item tooltip or status bar description.
The name will also appear in the View
menu items after you next re-boot SU.
You can cancel this dialogue to abort
the store operation.

4) Choose a 'Recall favourite' item from
the View menu or toolbox to restore a
previously stored appearance.

Choose 'Export' to export a bitmap of the
current model rendered using each of the
stored favourites.  Bitmaps are stored in
a sub-folder of the current model's file
folder.
Folder name is '<model name> Renders'
Each file uses the model name with the
favourite index and name appended.

'Set Options' lets you set the options
used when exporting files, and you can
also choose whether to recall complete
fog/shadow settings, or just their on/off
status.

Use the "Show settings"
item to see all of the current stored
favourites, and a listing of the currently
visible render settings.

You may use this plugin free of charge
subject to the condition that any
copyright notices remain intact.
The plugin is supplied 'as is' and no
warranty is implied - you use it at
you own risk.		
END
		
		UI.messagebox(string,MB_MULTILINE)
		return nil
	end
	
	# Set up user interface	
	unless file_loaded?('Trog_Renders.rb')
	
		# Make sub_menu and toolbar
		view_menu = UI.menu('View')
		view_menu.add_separator
		my_menu = view_menu.add_submenu('Render Favourites...')
		my_toolbar = UI::Toolbar.new("Render Favourites")
		
		# GUI items - Recall
		@@recall_cmds = []
		(0...MAX_RENDERS).each do |index|
			name = "#{index+1}"
			if render_exists?(index)
				name << ": #{@@renders[index][2]}"
			end
			item = UI::Command.new("Recall favourite #{index+1}"){self.render_load(index)}
			item.set_validation_proc{
			if self.render_exists?(index)
				MF_ENABLED
			else
				MF_GRAYED
			end}
			description = "Recall render #{name}"
			item.status_bar_text = description
			item.tooltip = description
			item.small_icon = File.join(RENDERS_DIR,"Recall_#{index+1}sm.png")
			item.large_icon = File.join(RENDERS_DIR,"Recall_#{index+1}lg.png")
			my_menu.add_item(item)
			my_toolbar = my_toolbar.add_item(item)
			@@recall_cmds[index] = item
		end
		
		my_menu.add_separator
		my_toolbar = my_toolbar.add_separator
		
		# GUI items - Store
		(0...MAX_RENDERS).each do |index|
			item = UI::Command.new("Store favourite #{index+1}"){self.render_store(index)}
			description = "Store render favourite #{index+1}"
			item.status_bar_text = description
			item.tooltip = description
			item.small_icon = File.join(RENDERS_DIR,"Store_#{index+1}sm.png")
			item.large_icon = File.join(RENDERS_DIR,"Store_#{index+1}lg.png")
			my_menu.add_item(item)
			my_toolbar = my_toolbar.add_item(item)
		end
		
		my_menu.add_separator
		my_toolbar = my_toolbar.add_separator
	
		# Export Images
		item = UI::Command.new("Export images"){self.renders_export}
		description = "Export a bitmap image for each render favourite"
		item.tooltip = description
		item.status_bar_text = description
		item.small_icon = File.join(RENDERS_DIR,'Export_sm.png')
		item.large_icon = File.join(RENDERS_DIR,'Export_lg,png')
		my_menu.add_item(item)
		my_toolbar = my_toolbar.add_item(item)
		
		# Show settings
		item = UI::Command.new("Show current renders"){self.show_renders}
		description = "Show stored favourites and current render settings"
		item.tooltip = description
		item.status_bar_text = description
		item.small_icon = File.join(RENDERS_DIR,'Show_info_sm.png')
		item.large_icon = File.join(RENDERS_DIR,'Show_info_lg.png')
		my_menu.add_item(item)
		my_toolbar = my_toolbar.add_item(item)
		
		# Show Options
		item = UI::Command.new("Set options..."){self.renders_plugin_opts}
		description = "Set up recall and image export options for render plugin"
		item.tooltip = description
		item.status_bar_text = description
		item.small_icon = File.join(RENDERS_DIR,'Options_sm.png')
		item.large_icon = File.join(RENDERS_DIR,'Options_lg.png')
		my_menu.add_item(item)
		my_toolbar = my_toolbar.add_item(item)
		
		# Load Options
		item = UI::Command.new("Save settings..."){self.render_opts_save}
		description = "Save all favourites and settings to a file"
		item.tooltip = description
		item.status_bar_text = description
		my_menu.add_item(item)
		
		# Save options
		item = UI::Command.new("Load settings..."){self.render_opts_load}
		description = "Load a complete set of favourites and settings"
		item.tooltip = description
		item.status_bar_text = description
		my_menu.add_item(item)
		
		# Show about (menu only)
		item = UI::Command.new("About..."){self.about_renders}
		description = "Information and help for render favourites plugin"
		item.tooltip = description
		item.status_bar_text = description
		my_menu.add_item(item)
		
		# Show the new toolbar unless it was hidden by the user
		# during the previous session
		if my_toolbar.get_last_state != 0
			my_toolbar.show
		else
			my_toolbar.hide
		end
			
	end
	
	file_loaded('Trog_Renders.rb')
	
end #module Trog